{----------------------------------------------------------------------------
  UNIT        : UVersionInfo
  ---------------------------------------------------------------------------
  DESCRIPTION : TVersionInfo retrieves the version info of a given executable
                or dynamic link library.
  ---------------------------------------------------------------------------
  STATUS      : Final
  ---------------------------------------------------------------------------
  VERSION     : 1.0
  LANGUAGE    : Delphi 5.0 Update Pack 1
  ---------------------------------------------------------------------------
  AUTHORS     : Daniel Rutten (DR)
  ---------------------------------------------------------------------------
  HISTORY     : 25.09.2001 - Created (DR)
----------------------------------------------------------------------------}

unit UVersionInfo;

interface

uses
  Classes;

type
  TVersionInfo = class(TComponent)
  private
    FCompanyName: string;
    FFileDescription: string;
    FFileVersion: string;
    FInternalName: string;
    FLegalCopyright: string;
    FLegalTradeMarks: string;
    FOriginalFilename: string;
    FProductName: string;
    FProductVersion: string;
    FComments: string;
  protected
  public
    constructor Create(AOwner: TComponent); override;
    function Execute(const AFilename: string): Boolean;
    property CompanyName: string read FCompanyName;
    property FileDescription: string read FFileDescription;
    property FileVersion: string read FFileVersion;
    property InternalName: string read FInternalName;
    property LegalCopyright: string read FLegalCopyright;
    property LegalTradeMarks: string read FLegalTradeMarks;
    property OriginalFilename: string read FOriginalFilename;
    property ProductName: string read FProductName;
    property ProductVersion: string read FProductVersion;
    property Comments: string read FComments;
  published
  end;

procedure Register;

implementation

uses
  SysUtils, Windows;

procedure Register;
begin
  RegisterComponents('VersionInfo', [TVersionInfo]);
end;

{ TVersionInfo }

constructor TVersionInfo.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Execute(ParamStr(0))
end;

function TVersionInfo.Execute(const AFilename: string): Boolean;
var
  MyHandle: Cardinal;
  MyVersionInfoBuffer: Pointer;
  MyVersionInfoSize: Cardinal;

  function GetVersionInfo(const Key: string): string;

    function SwapLong(L: LongInt): LongInt; assembler;
      asm rol eax, 16;
    end;

  var
    MyVersionValueSize: Cardinal;
    MyVersionValueBuffer: Pointer;
  begin
    VerQueryValue(MyVersionInfoBuffer, '\VarFileInfo\Translation', MyVersionValueBuffer, MyVersionValueSize);
    if VerQueryValue(MyVersionInfoBuffer, PChar(Format('\StringFileInfo\%.8x\%s', [SwapLong(LongInt(MyVersionValueBuffer^)), '\' + Key])), MyVersionValueBuffer, MyVersionValueSize) then
      Result := StrPas(PChar(MyVersionValueBuffer))
    else
      Result := ''
  end;

begin
  FCompanyName := '';
  FFileDescription := '';
  FFileVersion := '';
  FInternalName := '';
  FLegalCopyright := '';
  FLegalTradeMarks := '';
  FOriginalFilename := '';
  FProductName := '';
  FProductVersion := '';
  FComments := '';
  MyVersionInfoSize := GetFileVersionInfoSize(PChar(AFilename), MyHandle);
  Result := MyVersionInfoSize > 0;
  if Result then
  begin
    GetMem(MyVersionInfoBuffer, MyVersionInfoSize);
    if GetFileVersionInfo(PChar(AFilename), MyHandle, MyVersionInfoSize, MyVersionInfoBuffer) then
    begin
      FCompanyName := GetVersionInfo('CompanyName');
      FFileDescription := GetVersionInfo('FileDescription');
      FFileVersion := GetVersionInfo('FileVersion');
      FInternalName := GetVersionInfo('InternalName');
      FLegalCopyright := GetVersionInfo('LegalCopyright');
      FLegalTradeMarks := GetVersionInfo('LegalTradeMarks');
      FOriginalFilename := GetVersionInfo('OriginalFilename');
      FProductName := GetVersionInfo('ProductName');
      FProductVersion := GetVersionInfo('ProductVersion');
      FComments := GetVersionInfo('Comments')
    end;
    FreeMem(MyVersionInfoBuffer, MyVersionInfoSize)
  end
end;

end.

 
